import torch
import torch.nn as nn
import torch.nn.functional as F

import numpy as np

from .net_MSNet import MSNet
from .net_FSNet import FSNet
from .net_TPNet import TPNet


class SlowFastNet(nn.Module):
    def __init__(self, hidden_dim):
        super(SlowFastNet, self).__init__()
        self.hidden_dim = hidden_dim

        self.net_spatial1 = MSNet(self.hidden_dim)
        self.net_spatial2 = FSNet(self.hidden_dim)
        self.net_temporal = TPNet(self.hidden_dim)

    def forward(self, silho1, silho2, batch_frame1=None, batch_frame2=None):
        feat_id1 = self.net_spatial1(silho1, batch_frame1)
        feat_id2 = self.net_spatial2(silho1, batch_frame1)
        feat_id3 = self.net_temporal(silho2, batch_frame2)
        feat_id = torch.cat([feat_id1, feat_id2, feat_id3], dim=1)
        return feat_id, None


