import os
import random
import numpy as np
from copy import deepcopy

from utils.data_loader import load_data
from network.model import Model

def initialize_data(config, cache, is_train):
    print(' ==> Initializing data source ...')
    print('       DATASET NAME: {}'.format(config['data']['dataset_name']))
    print('       DATASET PATH: {}'.format(config['data']['dataset_path']))
    print('       DATASET PIXEL: {}'.format(config['data']['resolution']))
    data_source = load_data(cache, is_train, **config['data'])
    if is_train: 
        print(' ==> Loading training data ...')
        print('       TRAINING NUM: ', len(list(data_source.seqs_label_set)))
        print('       TRAINING IDs: ', sorted(list(data_source.seqs_label_set)))
        if cache: 
            data_source.load_all_data()
    else: 
        print(' ==> Loading testing data ...')
        print('       TESTING GALLERY NUM: ', len(list(data_source[0].seqs_label_set)))
        print('       TESTING GALLERY IDs: ', sorted(list(data_source[0].seqs_label_set)))
        print('       TESTING PROBE NUM: ', len(list(data_source[1].seqs_label_set)))
        print('       TESTING PROBE IDs: ', sorted(list(data_source[1].seqs_label_set)))
        if cache: 
            data_source[0].load_all_data()
            data_source[1].load_all_data()
    print(' ==> Data initialization complete.')
    return data_source

def initialize_model(config, data_source, is_train):
    print(' ==> Initializing model...')
    data_config = config['data']
    model_config = config['model']
    model_param = deepcopy(model_config)
    model_param['is_train'] = is_train
    model_param['data_source'] = data_source
    model_param['img_size'] = data_config['resolution']
    batch_size = int(np.prod(model_config['batch_size']))
    model_param['save_name'] = '_'.join(map(str,[model_config['model_name'], batch_size, model_config['hidden_dim']])) 
    model = Model(**model_param)
    print('       MODEL NAME: {}'.format(model_param['model_name']))
    print('       BATCH SIZE: {}'.format(model_param['batch_size']))
    print('       HIDDEN DIM: {}'.format(model_param['hidden_dim']))
    print('       LEARN RATE: {}'.format(model_param['lr']))
    print('       LOSS MARGIN: {}'.format(model_param['margin']))
    print('       TRIP TYPE: {}'.format(model_param['hard_or_full_trip']))
    print('       FRAME NUM: {}'.format(model_param['frame_num']))
    print(' ==> Model initialization complete.')
    return model

def initialization(config, cache, is_train):
    print(' ==> Initialzing...')
    os.environ["CUDA_VISIBLE_DEVICES"] = config["CUDA_VISIBLE_DEVICES"]
    data_source = initialize_data(config, cache, is_train)
    model = initialize_model(config, data_source, is_train)
    return model
